#-------------------------------------------------------------------------------
#
# Thomas Thomassen
# thomas[at]thomthom[dot]net
#
#-------------------------------------------------------------------------------

require 'fileutils'
require 'logger'
require 'stringio'
require 'tmpdir'

Sketchup.require "TT_SUbD/settings"


module TT::Plugins::SUbD
module Log

  APP_NAME = PLUGIN_NAME
  TEMP_FILE_PREFIX = 'subd-'.freeze

  @log_to_console = Settings.read('log_to_console', false)

  def self.log_to_console?
    @log_to_console ? true : false
  end

  # TT::Plugins::SUbD::Log.log_to_console = true
  def self.log_to_console=(value)
    bool = value ? true : false
    @log_to_console = bool
    Settings.write('log_to_console', bool)
  end


  # Location where we save error logs we keep:
  if Sketchup.platform == :platform_win
    path = ENV['APPDATA'].to_s.dup
    path.force_encoding('UTF-8') if path.respond_to?(:force_encoding)
    app_data = File.expand_path(path)
  else
    home = File.expand_path(ENV['HOME'].to_s)
    app_data = File.join(home, 'Library', 'Application Support')
  end
  if File.exist?(app_data)
    path = File.join(app_data, APP_NAME)
    @logs_path = File.join(path, 'Logs')
  else
    warn 'Unable to find AppData directory.'
    @logs_path = nil # Indicates error.
  end


  def self.path
    @logs_path
  end


  def self.open_location
    UI.openURL(@logs_path)
  end


  def self.create_log_file(logs_path)
    if logs_path.nil?
      # This will be caught by the rescue at the bottom.
      raise RuntimeError, 'No path to log files.'
    end

    # Make sure the destination directory exist.
    unless File.directory?(logs_path)
      FileUtils.mkdir_p(logs_path)
    end

    # Generates log files like: subd-20141106-2652-1dpjrlb.log
    log_basename = self.make_tmpname(TEMP_FILE_PREFIX, nil)
    log_filename = File.join(logs_path, "#{log_basename}.log")

    # Create the log file.
    mode = File::WRONLY | File::APPEND | File::CREAT
    log_file = File.open(log_filename, mode)

    # Make sure to close the file when the session terminates and delete it
    # if there was no errors.
    # This originally used Ruby's at_exit, but it appear that isn't triggering
    # under OSX for some reason. Which is bad since stuff in the StdLib, like
    # Tempfile, rely on that. We work around this for this extension by using
    # the AppObserver.onQuit event which calls Debug.end_log_to_file.

    log_file
  rescue
    # Account for permission failure and other errors. Fall back to logging
    # to a StringIO object. It won't be logged to file, but the extension can
    # still work.
    return StringIO.new
  end


  def self.end_log_to_file
    # Don't enable these debugs in release as it would mean a log file would
    # always be created.
    #self.debug "SketchUp closing..."
    #self.debug "> Error log: #{@log_file.path}"
    #self.debug "> Keep error log: #{self.keep_error_log?}"
    self.logger.close # This close the log IO stream - meaning out log file.
    if !self.keep_error_log? && File.exist?(@log_file.path)
      File.delete(@log_file.path)
    end
    nil
  end

  # @private
  def self.make_tmpname((prefix, suffix), n)
    prefix = (String.try_convert(prefix) or
              raise ArgumentError, "unexpected prefix: #{prefix.inspect}")
    suffix &&= (String.try_convert(suffix) or
                raise ArgumentError, "unexpected suffix: #{suffix.inspect}")
    t = Time.now.strftime("%Y%m%d")
    path = "#{prefix}#{t}-#{$$}-#{rand(0x100000000).to_s(36)}".dup
    path << "-#{n}" if n
    path << suffix if suffix
    path
  end


  # TODO: Clean up log folder after a given age?

  # Thread safe logger.
  # http://www.ruby-doc.org/stdlib-2.0.0/libdoc/logger/rdoc/Logger.html
  @logger = nil

  # Create an accessors that perform lazy initialization of the log file so
  # we only create a log file when the extension is ever being used.
  def self.logger
    @log_file ||= self.create_log_file(@logs_path)
    @logger ||= Logger.new(@log_file)
    @logger
  end


  # Keep a flag indicating if an error was experienced and that we should
  # keep the error log file.
  @keep_error_log = false

  def self.keep_error_log?
    @keep_error_log ? true : false
  end


  # http://www.ruby-doc.org/stdlib-2.0.0/libdoc/logger/rdoc/Logger/Severity.html
  #
  # DEBUG (0)
  # Low-level information, mostly for developers.
  #
  # INFO (1)
  # Generic (useful) information about system operation.
  #
  # WARN (2)
  # A warning.
  #
  # ERROR (3)
  # A handleable error condition.
  #
  # FATAL (4)
  # An unhandleable error that results in a program crash.
  #
  # UNKNOWN (5)
  # An unknown message that should always be logged.


  def self.debug(string)
    self.logger.debug {
      self.puts(string)
      string
    }
  end

  def self.error(string)
    @keep_error_log = true
    self.logger.error {
      self.puts(string)
      string
    }
  end

  def self.fatal(string)
    @keep_error_log = true
    self.logger.fatal {
      self.puts(string)
      string
    }
  end

  def self.info(string)
    self.logger.info {
      self.puts(string)
      string
    }
  end

  def self.warn(string)
    self.logger.warn {
      self.puts(string)
      string
    }
  end


  # Use this method to output debug info from code that might run in threads
  # other than the main one.
  #
  # If it's coming from a child thread the output
  # is stored in a variable on the thread itself which is queried by the
  # BackgroundTask manager when the thread completes.
  #
  # If this is called from the main thread the output is sent directly to
  # $stdout.
  def self.puts(*args)
    return false unless @log_to_console
    io = Thread.current[:stdout] || $stdout
    io.send(:puts, *args)
  end

  def self.p(*args)
    return false unless @log_to_console
    args.each { |arg| self.puts arg.inspect }
  end

end # module Log
end # module TT::Plugins::SUbD
