#-------------------------------------------------------------------------------
#
# Thomas Thomassen
# thomas[at]thomthom[dot]net
#
#-------------------------------------------------------------------------------

require 'json'
require 'net/http'
require 'rubygems'
require "uri"

Sketchup.require 'TT_SUbD/http/async_requests'
Sketchup.require 'TT_SUbD/logger'


module TT::Plugins::SUbD
  module Update

    def self.check(force: false)
      # Consider switching to HTTPS.
      # http://stackoverflow.com/a/12883718/486990

      # Only check once per session.
      return false if @checked && !force
      @checked = true

      server = Settings.read('Server', SERVER_URL)
      uri = URI.parse("#{server}/api/v1/product/#{PLUGIN_ID}/version")
      # Cannot rely on Net::Http working properly with HTTPS in previous
      # SketchUp versions.
      uri_http = uri.dup
      uri_http.scheme = 'http'
      uri_http.port = 80

      data = {
          :version => PLUGIN_VERSION,
          :license => License.read[:data],
          :sketchup => {
              :version => Sketchup.version,
              :bitness => POINTER_SIZE,
              :platform => PLATFORM_ID,
              :pro => Sketchup.is_pro?
          }
      }

      # noinspection RubyStringKeysInHashInspection
      headers = {
          'Content-Type' => 'application/json',
          # The following fields are used to indicate to Laravel that this is
          # an AJAX request and should get HTTP 422 and JSON response upon
          # validation errors.
          'Accept' => 'application/json',
          'X-Requested-With' => "SketchUp #{Sketchup.version}"
      }

      http = AsyncRequests.new
      http.on_timeout {
        Log.error "Check for #{PLUGIN_NAME} update timed out."
      }
      http.post(uri_http, data, headers) { |response|
        Log.debug "#{self} (Response)"
        Log.debug "#{response.class.inspect} (#{response.code})"
        if response.is_a?(Net::HTTPNoContent)
          Log.debug '> No updates available...'
        elsif response.is_a?(Net::HTTPSuccess)
          Log.debug response.body
          data = JSON.parse(response.body)
          self.handle(data)
        else
          Log.error '> Failed to check for update...'
          Log.error response.body
        end
      }
      true
    end

    def self.available?
      version = Settings.read('UpdateVersion')
      return false if version.nil?
      Gem::Version.new(version) > Gem::Version.new(PLUGIN_VERSION)
    end

    def self.version
      Settings.read('UpdateVersion') or raise
    end

    def self.timestamp
      Settings.read('UpdateTimestamp') or raise
    end

    # @param [Hash] data
    def self.handle(data)
      Settings.write('UpdateVersion', data['version'])
      Settings.write('UpdateTimestamp', data['timestamp'])
      nil
    end

  end # module Update
end # module TT::Plugins::SUbD
